<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:jmx="http://xml.kishou.go.jp/jmaxml1/"
xmlns:h="http://xml.kishou.go.jp/jmaxml1/informationBasis1/"
xmlns:m="http://xml.kishou.go.jp/jmaxml1/body/meteorology1/"
exclude-result-prefixes="xsl jmx h m">

  <xsl:import href='s0-ymdhn.xsl' />

  <!-- 注意報・警報電文から市町村等別発表状況の表（HTML）を作る例 -->
  <xsl:output method="html" encoding="UTF-8"/>

  <!-- entry point -->
  <xsl:template match="/">
    <!-- 不適切な情報種類の電文を与えた場合メッセージを出して終了する。
    xsltproc ならば異常終了する。
    xsltproc を使う場合このメッセージが標準出力に出てしまうため、
    -o オプションで出力ファイルを指定することを推奨。 -->
    <xsl:if test="/jmx:Report/h:Head/h:InfoKind != '気象警報・注意報'">
      <xsl:message terminate="yes">
        <xsl:value-of
        select="concat('bad InfoKind ', /jmx:Report/h:Head/h:InfoKind)"/>
      </xsl:message>
    </xsl:if>
    <!-- now that we are sure met warning document is input,
    let us start processing it. -->
    <xsl:apply-templates select="/jmx:Report" />
  </xsl:template>

  <!-- the root element of the document -->
  <xsl:template match="jmx:Report">
    <html>
      <head>
        <meta charset="utf-8"/>
        <title><xsl:call-template name="title" /></title>
        <style>
          span.m17nja { display: inline; }
          span.m17nen { display: none; }
        </style>
        <!-- CSS スタイルシートで色付けなどができます -->
        <link rel="stylesheet" type="text/css" href="style.css"/>
        <script>
          function m17nen() {
            document.styleSheets[0].cssRules[0].style.cssText = 'display: none;';
            document.styleSheets[0].cssRules[1].style.cssText = 'display: inline;';
          }
          function m17nja() {
            document.styleSheets[0].cssRules[0].style.cssText = 'display: inline;';
            document.styleSheets[0].cssRules[1].style.cssText = 'display: none;';
          }
        </script>
      </head>
      <body>
        <!-- 本物の気象庁ホームページと間違えないための注記 -->
        <p>テスト用スタイルシート</p>
        <h1><xsl:call-template name="title" /></h1>
        <!-- 一意にできるタグには @id, それが無理なものには @class を
        なるべくつけている。あとで CSS でいじるときに追加するのも面倒だし、
        公共サービスとしてスクレイピングが容易になるよう配慮しておくべき。 -->
        <p id="credit">
          <!-- PublishingOffice が責任をもつ官署。
          EditorialOffice はその名と無関係に府県予報区を指示するものなので
          ここでは出番はない -->
          <span id="PublishingOffice">
            <xsl:value-of select="/jmx:Report/jmx:Control/jmx:PublishingOffice"/>
          </span>
          <xsl:text> </xsl:text>
          <span id="ReportDateTime">
            <xsl:call-template name="nengappijihun">
              <xsl:with-param name="t" select="/jmx:Report/h:Head/h:ReportDateTime"/>
            </xsl:call-template>
          </span>
          <!-- 訂正報はそのむねカッコ書きすることにした。
          訂正報といえども ReportDateTime に発表と呼ぶべきと考えられる。 -->
          <xsl:text> 発表</xsl:text>
          <xsl:if test="/jmx:Report/h:Head/h:InfoType != '発表'">
            <xsl:value-of select="concat(' (', /jmx:Report/h:Head/h:InfoType, ')')"/>
          </xsl:if>
          <input type="button" value="&#x1F1EF;&#x1F1F5;" onclick="m17nja()" />
          <input type="button" value="&#x1F1EC;&#x1F1E7;" onclick="m17nen()" />
        </p>
        <table id="datatable">
          <tr class="tablehead">
            <th class="area">市町村等</th>
            <th class="warnstate">発表中の注意報・警報</th>
            <th class="remark">付加事項・切り替え予告</th>
          </tr>
          <!-- m:Warning を選択するのに
          m:Warning[@type='気象警報・注意報（市町村等）']
          として選択することも可能だが、 @type の文字列については
          辞書でのコミットがなく、将来変更があるかもしれないので、
          さしあたり安定して動作する方法として m:Warning[last()] を使った.  -->
          <xsl:apply-templates select="/jmx:Report/m:Body/m:Warning[last()]/m:Item"
          />
        </table>
      </body>
    </html>
  </xsl:template>
  <!-- end jmx:Report -->

  <!-- path relative to "/jmx:Report/m:Body/m:Warning[last()]/m:Item" -->
  <xsl:template match="m:Item">
    <tr>
      <td class="area" id="area{m:Area/m:Code}">
        <a href="https://www.jma.go.jp/jp/warn/f_{m:Area/m:Code}.html">
          <xsl:value-of select="m:Area/m:Name"/>
        </a>
      </td>
      <td class="warnstate">
        <xsl:for-each select="m:Kind">
          <!-- 小技： position() &gt; 1 はリスト文字列化に読点を
          いれこむテクニック。 -->
          <xsl:if test="position() &gt; 1">
            <xsl:text>、</xsl:text>
          </xsl:if>
          <xsl:call-template name="display-chukeiho-name" />
        </xsl:for-each>
      </td>
      <td class="remark">
        <xsl:if test="m:Kind/m:Attention">
          <xsl:text>付加事項：</xsl:text>
          <xsl:for-each select="m:Kind/m:Attention/m:Note">
            <xsl:if test="position() &gt; 1">
              <xsl:text>、</xsl:text>
            </xsl:if>
            <xsl:call-template name="display-fukajiko" />
          </xsl:for-each>
          <xsl:text>。</xsl:text>
        </xsl:if>
        <xsl:apply-templates select="m:Kind/m:WarningNotice" />
      </td>
    </tr>
  </xsl:template>
  <!-- end m:Item -->

  <xsl:key name="warnname" match="term" use="c" />

  <!-- path relative to m:Item/m:Kind -->
  <xsl:template name="display-chukeiho-name">
    <xsl:variable name="code" select="m:Code" />
    <xsl:variable name="ename">
      <xsl:for-each select="document('warnlevels.xml')">
        <xsl:value-of select="key('warnname', $code)/w[@lang='en']" />
        <xsl:text> </xsl:text>
        <xsl:value-of select="key('warnname', $code)/lev[@lang='en']" />
      </xsl:for-each>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="m:Name">
        <span class="kind{$code}">
          <xsl:choose>
            <xsl:when test="$ename != ' '">
              <span class="m17nja"><xsl:value-of select="m:Name" /></span>
              <span class="m17nen"><xsl:value-of select="$ename" /></span>
            </xsl:when>
            <!-- failsafe -->
            <xsl:otherwise>
              <xsl:value-of select="m:Name" />
            </xsl:otherwise>
          </xsl:choose>
          <xsl:if test="m:Condition">
            <xsl:value-of select="concat('（', m:Condition, '）')"/>
          </xsl:if>
        </span>
      </xsl:when>
      <xsl:when test="m:Status = '発表警報・注意報はなし'">
        <span class="clear">
        <span class="m17nja">なし</span>
        <span class="m17nen">none</span>
        </span>
      </xsl:when>
      <xsl:otherwise>
        <!-- unexpected - just for failsafe -->
        <span class="unknown">
          <xsl:value-of select="m:Status"/>
        </span>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- path relative to m:Kind/m:Attention/m:Note -->
  <xsl:template name="display-fukajiko">
    <!-- タグを開いた直後で xsl:attribute を使うと
    そのタグに属性をつけることができる。
    -->
    <span>
      <xsl:choose>
      <xsl:when test="contains(., '警戒')">
        <xsl:attribute name='class'>wrn</xsl:attribute>
      </xsl:when>
      <xsl:when test="contains(., '注意')">
        <xsl:attribute name='class'>adv</xsl:attribute>
      </xsl:when>
      </xsl:choose>
      <xsl:value-of select="."/>
    </span>
  </xsl:template>

  <xsl:template match="m:WarningNotice">
    <span>
      <xsl:choose>
      <xsl:when test="contains(m:Note, '特別警報')">
        <xsl:attribute name='class'>emg</xsl:attribute>
      </xsl:when>
      <xsl:when test="contains(m:Note, '警報')">
        <xsl:attribute name='class'>wrn</xsl:attribute>
      </xsl:when>
      </xsl:choose>
      <!-- 「までに」はウェブサイト実例による。 -->
      <xsl:value-of select="concat(m:StartTime, 'までに', m:Note)" />
    </span>
    <xsl:text>。</xsl:text>
  </xsl:template>

  <!-- path relative to /jmx:Report -->
  <!-- 試験電文など非通常であればその旨注記する -->
  <xsl:template name="title">
    <xsl:value-of select="h:Head/h:Title"/>
    <xsl:if test="jmx:Control/jmx:Status != '通常'">
      <xsl:value-of select="concat(' (', jmx:Control/jmx:Status, ')')"/>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
